/*
 * Blue Heat/Net backwards compatability code
 */

#include "compat.h"

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16))
int tty_insert_flip_string(struct tty_struct *tty, unsigned char *chars,
								size_t size)
{
	memcpy(tty->flip.char_buf_ptr, chars, size);
	tty->flip.char_buf_ptr += size;
	memset(tty->flip.flag_buf_ptr, 0, size);
	tty->flip.flag_buf_ptr += size;
	tty->flip.count += size;

	return size;
}

int get_receive_room(struct tty_struct *tty)
{
	struct tty_ldisc *ldisc;
	int count = 0;

	ldisc = tty_ldisc_ref(tty);
	if (!ldisc)
		goto out;
	if (!ldisc->receive_room) {
		tty_ldisc_deref(ldisc);
		goto out;
	}

	count = ldisc->receive_room(tty);
	tty_ldisc_deref(ldisc);

out:
	return count;
}
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11))
#include <linux/sched.h>
#include <asm/errno.h>
unsigned long wait_for_completion_interruptible_timeout(struct completion *x,
							unsigned long timeout)
{
	spin_lock_irq(&x->wait.lock);
	if (!x->done) {
		DECLARE_WAITQUEUE(wait, current);

		wait.flags |= WQ_FLAG_EXCLUSIVE;
		__add_wait_queue_tail(&x->wait, &wait);
		do {
			if (signal_pending(current)) {
				timeout = -ERESTARTSYS;
				__remove_wait_queue(&x->wait, &wait);
				goto out;
			}
			__set_current_state(TASK_INTERRUPTIBLE);
			spin_unlock_irq(&x->wait.lock);
			timeout = schedule_timeout(timeout);
			spin_lock_irq(&x->wait.lock);
			if (!timeout) {
				__remove_wait_queue(&x->wait, &wait);
				goto out;
			}
		} while (!x->done);
		__remove_wait_queue(&x->wait, &wait);
	}
	x->done--;
out:
	spin_unlock_irq(&x->wait.lock);
	return timeout;
}
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8))
#include <asm/uaccess.h>
int kernel_sendmsg(struct socket *sock, struct msghdr *msg, struct kvec *vec,
							size_t num, size_t size)
{
	mm_segment_t oldfs = get_fs();
	int result;

	set_fs(KERNEL_DS);
	/*
	 * the following is safe, since for compiler definitions of kvec and
	 * iovec are identical, yielding the same in-core layout and alignment
	 */
	msg->msg_iov = (struct iovec *)vec;
	msg->msg_iovlen = num;
	result = sock_sendmsg(sock, msg, size);
	set_fs(oldfs);

	return result;
}

int kernel_recvmsg(struct socket *sock, struct msghdr *msg, struct kvec *vec,
					size_t num, size_t size, int flags)
{
	mm_segment_t oldfs = get_fs();
	int result;

	set_fs(KERNEL_DS);
	/*
	 * the following is safe, since for compiler definitions of kvec and
	 * iovec are identical, yielding the same in-core layout and alignment
	 */
	msg->msg_iov = (struct iovec *)vec;
	msg->msg_iovlen = num;
	result = sock_recvmsg(sock, msg, size, flags);
	set_fs(oldfs);

	return result;
}
#endif
