/*
 * Blue Heat/Net backwards compatability code
 */

#ifndef _BHN_COMPAT_H
#define _BHN_COMPAT_H

#include <linux/version.h>

#ifndef KERNEL_VERSION
#error "KERNEL_VERSION macro missing. Please fix me."
#endif

#define MIN(a, b)	({						\
		int a1 = (a);						\
		int b1 = (b);						\
		a1 < b1 ? a1 : b1;					\
	})

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20))
#define STRUCT_TERMIOS	struct termios
#else
#define STRUCT_TERMIOS	struct ktermios
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16))
#define UNPROCESSED_FLIP_BUFFER(tty)	(tty->flip.count)
#elif (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18))
#define UNPROCESSED_FLIP_BUFFER(tty)	(tty->buf.head)
#else
#define UNPROCESSED_FLIP_BUFFER(tty)	(tty->buf.head && tty->buf.head->read != tty->buf.head->commit)
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,16))
#define MIN_WITH_FLIPBUF(count)		MIN(count, TTY_FLIPBUF_SIZE)
#include <linux/tty.h>
extern int get_receive_room(struct tty_struct *tty);
extern int tty_insert_flip_string(struct tty_struct *tty, unsigned char *chars,
								size_t size);
#else
#define MIN_WITH_FLIPBUF(count)		(count)
#define get_receive_room(tty)	(tty)->receive_room
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11))
#include <linux/completion.h>
extern unsigned long wait_for_completion_interruptible_timeout(
				struct completion *x, unsigned long timeout);
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9))
#define tty_ldisc_ref(tty)	(&(tty)->ldisc)
#define tty_ldisc_deref(ld)
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,8))
#define kvec iovec

#include <linux/socket.h>
#include <linux/net.h>
extern int kernel_sendmsg(struct socket *sock, struct msghdr *msg,
				struct kvec *vec, size_t num, size_t size);
extern int kernel_recvmsg(struct socket *sock, struct msghdr *msg,
			struct kvec *vec, size_t num, size_t size, int flags);
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,6))
#define sock_create_kern	sock_create
#endif

#endif
