/*
 *
 *  The Blue Heat/Net driver's mini network interface
 *
 *  Copyright (C) 2005 Connect Tech Inc.
 *
 *  Aug 06 2005 Stuart MacDonald <stuartm@connecttech.com>
 *	First draft.
 *
 */

#include <linux/net.h>
#include "compat.h"

#include <BHN_flash.h>
#include <CDS.h>
#include <BHN_protocol.h>
#include "bhn.h"

/* Send data on the network */
int sendto(struct socket *sock, void *buf, size_t size, unsigned flags,
					struct sockaddr *addr, int addr_len)
{
	struct msghdr mh;
	struct kvec kv;

	mh.msg_name = addr;
	mh.msg_namelen = addr_len;
	mh.msg_control = NULL;
	mh.msg_controllen = 0;
	mh.msg_flags = flags;

	kv.iov_base = buf;
	kv.iov_len = size;

	return kernel_sendmsg(sock, &mh, &kv, 1, size);
}

int send(struct socket *sock, void *buf, size_t size, unsigned flags)
{
	return sendto(sock, buf, size, flags, NULL, 0);
}

/* Receive network data */
int recvfrom(struct socket *sock, void *buf, size_t size, unsigned flags,
					struct sockaddr *addr, int addr_len)
{
	struct msghdr mh;
	struct kvec kv;

	mh.msg_name = addr;
	mh.msg_namelen = addr_len;
	mh.msg_control = NULL;
	mh.msg_controllen = 0;
	mh.msg_flags = flags;

	kv.iov_base = buf;
	kv.iov_len = size;

	return kernel_recvmsg(sock, &mh, &kv, 1, size, flags);
}

int recv(struct socket *sock, void *buf, size_t size, unsigned flags)
{
	return recvfrom(sock, buf, size, flags, NULL, 0);
}
